#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os
from time import localtime, strftime
import logging

import sqlite3


"""
this module is created ONLY for SQLite3 through Python binding.
it uses SQL valid only for SQLite3.
"""

__author__="Balzano Alessandro, aka delta aka alfateam aka alfateam123"
__license__="Creative Commons V3.0 BY-SA-NC"

#-----------------
#impostazioni per logging
LOG_FILENAME='PyPMMLog2.txt'
logging.basicConfig(filename=LOG_FILENAME, level=logging.DEBUG)
#-----------------


# -------------------- Model Class ----------------------------------------

class Model(object):
    """ the Model """
    DBPATH=os.path.join(
                os.path.dirname(os.path.abspath(sys.argv[0])),"db.db")
    def __init__(self, dbPath=None):
        if dbPath: self.DBPATH=dbPath
        logging.debug('Model:dbPath== '+self.DBPATH)
        self.connection=sqlite3.connect(self.DBPATH)
        self.cursor=self.connection.cursor()
        try:
           self.cursor.execute('create table if not exists movement_table (id \
integer primary key, value float, causal text varchar, \
notes text varchar, direction text varchar, day integer,\
month integer, year integer)')
        except:
            #logging.debug("Model:"+", ".join([str(exc_value) for exc_value in sys.exc_info()]))
            print "Model: %s"%sys.exc_info()
        #creating some constants
        self.VALUE, self.CAUSAL, self.NOTES, self.DIRECTION, self.DAY, self.MONTH, self.YEAR=range(1, 8)

    def get_latest_id(self, cursor=None):
        if not cursor: cursor=self.cursor
        records=cursor.execute('select * from movement_table').fetchall()
        return (not records and -1) or max(int(record[0]) for record in records)

    def add_movement(self, value, causal, notes, direction, date):
        day, month, year=(int(item) for item in date.split('/'))
        self.cursor.execute('insert into movement_table values (?, ?, ?, ?, ?, ?, ?, ?)',(
        self.get_latest_id(self.cursor)+1, value, causal, notes, direction, day, month, year))
        #logging.debug('Model:object added <value=%s, causal=%s, notes=%s, \
        #direction=%s, date=%s>'%(value, causal, notes,\
        #                       direction, day, month, year))
        self.connection.commit()

    def get_movement(self, value='*', causal='*', notes='*',direction='*', \
    year='*', month='*', day='*'):
        """
        questa funzione non va.
        """
        execute_string, parameters='select * from movement_table', []
        values=filter(lambda elem: elem[-1]!='*', [('value',value),
         ('causal',causal), ('notes',notes),
        ('direction', direction), ('year', year), ('month', month), ('day',day)])
        for param in values:
            parameters.append(param[-1])
            if not execute_string.count('where'):
                execute_string=' '.join([execute_string, "where"])
            if param[0]!=values[-1][0]: #the last
                execute_string=' '.join([execute_string, param[0], "=? and" ])
            else:
                execute_string=' '.join([execute_string, param[0], "=?" ])
        #print execute_string, causal, month, year, day
        records=self.cursor.execute(execute_string, parameters).fetchall()

##        # I tried to improve the code...
##        #"select * from movement_table"+(item[-1]!='*' and "where %s=?"%item[0]
##        parameters=[('value',value), ('causal',causal), ('notes',notes),
##        ('direction', direction), ('day', day), ('month', month), ('year', year)]
##
##        queries=["select * from movement_table "+
##            ((item[-1]!='*' and "where %s=?"%item[0]) or "")
##            for item in parameters]
##
##        results=[]
##        for pos in xrange(len(parameters)):
##            if parameters[pos][-1]!="*":
##                results.append(self.cursor.execute(queries[pos], (parameters[pos][-1], )).fetchall())
##            else:
##                results.append(self.cursor.execute(queries[pos]).fetchall())
##
##        #I transform all in intersection of sets.
##        records_set=set(results[0]).intersection(results[1]).intersection(results[2]).intersection(results[3]).intersection(results[4]).intersection(results[5]).intersection(results[6])
##        #I transform records_set in a list using filter
##        records=filter(None, records_set)
##        #logging.debug('Model: records are '+str(len(records)))

        return records

    def get_movement_by_id(self, movement_id):
        records=self.cursor.execute('select * from \
         movement_table where id=?',
        (movement_id,)).fetchall()
        #logging.debug('Model: records are '+str(len(records)))
        return records

    def remove_movement(self, movement_id):
        return [record for record in self.cursor.execute('delete * from \
            movement_table where id=?',
        (movement_id))]

    def get_last_movement(self):
        """
        this function gets the date of the cronologically last inserted movement.
        If no movements are found, the function returns None.
        """
        movs=self.cursor.execute("select day, month, year from movement_table").fetchall()
        try:
            lastYear=max(i[2] for i in movs)
            movs_year_filter=filter(lambda x: x[2]==lastYear, movs)

            lastMonth=max(i[1] for i in movs_year_filter)
            movs_month_filter=filter(lambda x: x[1]==lastMonth, movs_year_filter)

            lastDay=max(i[0] for i in movs_month_filter)

            #only the first movement of the filtered sequence
            last_movement=filter(lambda x: x[0]==lastDay, movs_month_filter)[0]

        except ValueError:
            #max arg is an empty sequence
            last_movement=None
            pass

        return last_movement

    def update(self, movement_id, column_name, new_value):
        """ Internal. """
        self.cursor.execute(
            "update movement_table set %s = ? where id = ?"%column_name,
            (new_value, movement_id, )
            )
        self.connection.commit()

    def update_movement(self, movement_id, newValuesDict):
        """
        usage:
        movement_id is the ID of the movement you want to modify.
        newValuesDict is a dict, its keys are column's name of the table,
            its values the new values.

        Looks at tests to say correct usage.
        """
        for newVal in newValuesDict:
            column_name, new_value=newVal, newValuesDict[newVal]
            self.update(movement_id, column_name, new_value)


    #def set_movement(self, movement_id, args):
        #""" this method is not finished """
        #set_function={'value': lambda mov_id, value: self.store.find(self.Movement,\
        #self.Movement.id==mov_id).value(value)}

    def close(self,  commit=True):
        if commit: self.connection.commit()
        self.connection.close()

# ----------------------------- UNITTEST -------------------------------------

if __name__=="__main__":
    import unittest
    from controller import Controller
    class Test_Model_on_Memory(unittest.TestCase):
        def setUp(self):
            self.model=Model(':memory:')
            #self.model2=Model('C:/projects/PyPMM/pypmm/db.db')
            self.contr=Controller()
            self.added_movements=0
            #format: value, causal, notes, direction, date(day/month/year)
            movements=[[33.50, u'Spese extra', u'Wired', u'Expense', '2','5','2010'],
            [5, u'Spese extra', u'pantalone', u'Expense', '12','12','1999'],
            [10, u'Altre entrate', u'rimborso benzina', u'Income', '15','8','2009'],
            [1000, u'Stipendio', u'stipendio', u'Income', '2', '5', '2010'],
            ]
            for element in movements:
                #we use "service" functions because self.contr.add_movement adds the movement directly
                #and we need to add movements by ourself
                if self.contr.is_movement_correct(element[0], element[1], element[2], element[3], "/".join(element[4:])):
                    self.added_movements+=1
                    self.model.add_movement(element[0], element[1], element[2], element[3], "/".join(element[4:]))

        def test_get_latest_id(self):
            self.assertEqual(self.model.get_latest_id(self.model.cursor), 3)
            self.assertEqual(self.model.get_latest_id(self.model.cursor)+1, 4)
            self.assertNotEqual(self.model.get_latest_id(self.model.cursor)+1, 3)

        def test_get_movement(self):
            self.assertEqual(len(self.model.get_movement()), self.added_movements)
            self.assertEqual(len(self.model.get_movement(direction=u'Income')), 2)
            self.assertEqual(len(self.model.get_movement(direction=u'Expense')), 2)
            self.assertEqual(len(self.model.get_movement(causal=u'Supermercato')), 0)
            self.assertEqual(len(self.model.get_movement(direction=u'Income', causal=u'Altre entrate')), 1)
            self.assertEqual(len(self.model.get_movement(direction=u'Expense', causal=u'Altre entrate')), 0)
            self.assertEqual(len(self.model.get_movement(direction=u'Expense', value=5)), 1)
            self.assertEqual(len(self.model.get_movement(direction=u'Expense', value=33.50)), 1)
            self.assertEqual(len(self.model.get_movement(direction=u'Expense', day=2)), 1)
            self.assertEqual(len(self.model.get_movement(direction=u'Income', day=2)), 1)
            self.assertEqual(len(self.model.get_movement(month=5, day=2, year=2010)), 2)
            self.assertEqual(len(self.model.get_movement(year=2010, month='*')), 2)
            self.assertEqual(len(self.model.get_movement(year=2010, month=u'*')), 2)
            self.assertEqual(len(self.model.get_movement(month=5, year=2010, causal="Altre entrate")), 0)
            self.assertEqual(len(self.model.get_movement(month=5, year=2010, causal="Stipendio")), 1)

        def test_model_constants(self):
            self.assertEqual(self.model.VALUE, 1)
            self.assertEqual(self.model.DIRECTION, 4)

        def test_get_movement_by_id(self):
            #value of movement with id=2 is 10
            self.assertEqual(self.model.get_movement_by_id(2)[0][self.model.VALUE], 10)
            #direction of movement with id=2 is not Expense
            self.assertNotEqual(self.model.get_movement_by_id(2)[0][self.model.DIRECTION], u'Expense')
            #a movement  with id=8 does not exists
            self.assertEqual(self.model.get_movement_by_id(8), [])

        def test_get_last_movement(self):
            self.assertEqual(self.model.get_last_movement(), (2, 5, 2010))

        def test_update_movement(self):
            self.model.update_movement(3, {'value':580, 'notes':u"stip"})
            self.assertEqual(self.model.get_movement_by_id(3)[0][self.model.VALUE], 580)
            self.assertNotEqual(self.model.get_movement_by_id(3)[0][self.model.VALUE], 1000)
            self.assertEqual(self.model.get_movement_by_id(3)[0][self.model.NOTES], "stip")
            self.assertNotEqual(self.model.get_movement_by_id(3)[0][self.model.NOTES], "stipendio")
            self.assertEqual(self.model.get_movement_by_id(3)[0][self.model.DIRECTION], u"Income")
            self.assertNotEqual(self.model.get_movement_by_id(3)[0][self.model.DIRECTION], u"Expense")
        """
        #written when I was angry because nothing on my life went good
        def test_get_movement_with_model2(self):
            #the requested item exists
            self.assertEqual(len(self.model2.get_movement(causal="Gasolio", month=5, year=2010)), 1)
            self.assertEqual(self.model2.get_latest_id(), 1)
            self.assertEqual(self.model2.get_movement(causal="Gasolio", month=5, year=2010), self.model2.get_movement_by_id(1))
            self.assertNotEqual(self.model2.get_movement_by_id(1), [])
            self.assertEqual(self.model2.get_movement_by_id(1)[0][self.model2.DAY], 12)
            self.assertEqual(self.model2.get_movement_by_id(1)[0][self.model2.YEAR], 2010)
            self.assertNotEqual(self.model2.get_movement(year=2009) and True or False, True)
            self.assertEqual(self.model2.get_movement(causal="Gasolio", month=5, year=2010)[0][self.model2.VALUE], 20)
            self.assertEqual(self.model2.get_movement(causal="Gasolio", month=5, year=2010)[0][self.model2.DAY], 12)
        """
    #Ok, we can run the unittest!
    unittest.main()

