#!/usr/bin/env python
#-*- coding:utf-8 -*-

#STL modules
import Tkinter as tk
import tkMessageBox
import os
import sys
import datetime
#custom modules
from models import Model
from controller import Controller


"""
TODO:
>>modify the update form
>>insert a delete form
>>insert a stats switcher form
 -histograms with expense of a month
 -histograms with expense of a year
"""

class View(object):
    def __init__(self, master, controller, model):
        #creating the frame
        self.frame=tk.Frame(master)
        self.frame.pack()
        self.menubar=tk.Menu(self.frame)
        master.config(menu=self.menubar)
        self.controller, self.model=controller, model
		#first cascade: "manage movements"
        self.movements_menu=tk.Menu(self.menubar)
        self.menubar.add_cascade(label="manage movements", menu=self.movements_menu)

        self.movements_menu.add_command(label="aggiungi un\'entrata",
            command=lambda : AddIncomeForm(self.frame, controller, self))

        self.movements_menu.add_command(label="aggiungi una uscita",
            command=lambda : AddExpenseForm(self.frame, controller, self))

        self.movements_menu.add_command(label="cerca un movimento",
            command=lambda: SearchMovement(tk.Toplevel(self.frame), controller)
            )

        self.movements_menu.add_command(label="osserva quando hai inserito l'ultimo movimento",
            command=lambda : LastMovementForm(self.controller))

        self.movements_menu.add_command(label="modifica un movimento",
            command=lambda : SearchUpdateForm(self.frame, self.model))

        self.movements_menu.add_command(label="esci", command=self.frame.quit)

        #second cascade: "information"
        self.information_menu=tk.Menu(self.menubar)
        self.menubar.add_cascade(label="information", menu=self.information_menu)
        self.information_menu.add_command(label="About PyPMM",
                command=self.aboutPyPMM)

        #and now... showing the annual plan! as known as the form!

        self.table=DataTableInForm(self.frame, 13)
        self.table.addRow(["Causal", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"])
        try:
            expenses=[exp.splitlines()[0].strip(" ") for exp in open(self.controller.EXPENSE_PATH)]
            incomes=[exp.splitlines()[0].strip(" ") for exp in open(self.controller.INCOME_PATH)]
        except IOError:
            tkMessageBox.showerror(title="errore:",
				message="errore: alcuni file non sono stati trovati, l'applicazione non e\' utilizzabile.")
            sys.exit(1)
        year=datetime.date.today().year
        for causal in expenses+incomes:
            list_of_values=[causal]
            for month in xrange(1, 13):
                movements=self.controller.get_movement(causal=causal, month=month, year=year)
                sommator=sum(mov[self.controller.VALUE] for mov in movements)
                list_of_values.append(sommator)
                #print sum(mov[self.controller.VALUE] for mov in movements), "causal=%s, month=%s"%(causal, month)
            self.table.addRow(list_of_values)


    def aboutPyPMM(self):
        msg="PyPMM is a personal finance software.\nIt is written in Python with the Tkinter GUI library.\nIts creator is Alessandro Balzano, also known as \'delta\' or \'alfateam123\'\n(alfateam123@gmail.com or twitter.com\deltainf).\nIts source code is available for free, under GNU GPL"
        tkMessageBox.showinfo("About PyPMM", msg)

#------------- DataForm class ----------------------------------

class DataForm(object):
    """
    questa classe � stata creata con lo scopo
    di visualizzare testi (non modificabili) immagazzinati in tabelle.
    """
    def __init__(self, master, itemsNumber):
        """
        costruttore:
        master � una istanza di Tkinter.Tk
        itemsNumber � il numero di valori da immagazzinare nella tabella
        """

        self.labels=[]
        self.master=tk.Toplevel(master)
        self.itemsNumber=itemsNumber

    def createLabel(self, row, column, text="***"):
        """
        INTERNAL!
        """
        self.labels.append(tk.Label(self.master, text=text))
        self.labels[-1].grid(row=row, column=column)

    def addRow(self, values):
        """
        Questa funzione permette di aggiungere righe.
        """
        if len(values)!=self.itemsNumber:
            raise IndexError('items number is wrong. itemsNumber=%i, len(values)=%i'%(self.itemsNumber, len(values)))
        row=(len(self.labels)/self.itemsNumber)
        #fixed issue #3
        if len(self.labels): row+=1
        for item in zip(values, xrange(1, len(values)+1)):
            self.createLabel(row=row, column=item[1], text=unicode(item[0]))


class DataTableInForm(DataForm):
    def __init__(self, master, itemsNumber):
        self.labels=[]
        self.master=master
        self.itemsNumber=itemsNumber

#------------- AddMovementForm class ----------------------------------

class AddMovementForm(object):
    """
    this class lets user insert a movement in the database.
    """
    CAUSALS_FILE_PATH=""
    IS_EXPENSE_FORM=True
    def __init__(self, master, controller, view):
        """
        to insert a movement, the program needs some infos
        >value
        >direction (Expense/Income) (listbox)
        >day, month, year. These infos are taken by an unique entry with a regex
        dd/mm/yyyy
        >causal (listbox)
        >notes
        """
        self.controller, self.view=controller, view
        self.toplevel=tk.Toplevel(master)

        self.valueLabel=tk.Label(self.toplevel, text="inserisci valore del movimento")
        self.valueLabel.pack()
        self.valueEntry=tk.Entry(self.toplevel)
        self.valueEntry.pack()

        self.dateLabel=tk.Label(self.toplevel, text="inserisci la data del movimento\nformato: dd/mm/yyyy")
        self.dateLabel.pack()
        self.dateEntry=tk.Entry(self.toplevel)
        self.dateEntry.pack()

        self.causalsLabel=tk.Label(self.toplevel,
            text="clicca la causale che ti interessa.")
        self.causalsLabel.pack()
        self.causalsListbox=tk.Listbox(self.toplevel)
        self.causalsListbox.pack()
        #self.chargeCausals() #carica le causali


        self.notesLabel=tk.Label(self.toplevel, text="inserisci, se vuoi,\ndelle note sul movimento")
        self.notesLabel.pack()
        self.notesEntry=tk.Entry(self.toplevel)
        self.notesEntry.pack()

        self.submitButton=tk.Button(self.toplevel, text="aggiungi movimento", command=self.submitMovement)
        self.submitButton.pack()
        #questa variabile serve per mantenere l'indice della listbox causalsListbox in cui � indicato la direzione del movimento
        self.causalIndex='*'

        self.pollCausal()

    #solution found at http://www.java2s.com/Code/Python/GUI-Tk/GetselectedvaluefromListbox.htm
    def pollCausal(self):
        self.causalsListbox.after(200, self.pollCausal)
        try:
            self.causalIndex=int(self.causalsListbox.curselection()[0])
        except:
            pass
        #print self.causalIndex

    def submitMovement(self):
        date=str(self.dateEntry.get())
        value=float(self.valueEntry.get())
        notes=unicode(self.notesEntry.get())

        direction=self.IS_EXPENSE_FORM and u'Expense' or u'Income'

        #ricavo tutte le causali
        causalPosition=self.causalIndex

        try:
            causals_file=open(self.CAUSAL_FILE_PATH, 'r')
            causal=unicode([causal.splitlines()[0].strip(' ') for causal in causals_file][causalPosition])
            try:
                self.controller.add_movement(value, causal, notes, direction, date)
                tkMessageBox.showinfo(title='aggiunta dati', message='dati aggiunti con successo')
            except ValueError:
                tkMessageBox.showwarning(title='aggiunta dati', message='errore nei dati inseriti. Ricontrollare')
        except IndexError:
            tkMessageBox.showwarning('errore scelta causale causal=%i'%causalPosition,'non hai scelto la causale! clicca sulla lista dove sono elencate le causali')


class AddExpenseForm(AddMovementForm):
    def __init__(self, master, controller, view):
        AddMovementForm.__init__(self, master, controller, view)
        self.CAUSAL_FILE_PATH=self.controller.EXPENSE_PATH
        expense_causals=[causal.splitlines()[0] for causal in open(self.controller.EXPENSE_PATH)]
        for causal in expense_causals:
            self.causalsListbox.insert(tk.END, causal)

class AddIncomeForm(AddMovementForm):
    def __init__(self, master, controller, view):
        AddMovementForm.__init__(self, master, controller, view)
        self.CAUSAL_FILE_PATH=self.controller.INCOME_PATH
        income_causals=[causal.splitlines()[0] for causal in open(self.controller.INCOME_PATH)]
        for causal in income_causals:
            self.causalsListbox.insert(tk.END, causal)


#------------- SearchMovement class ----------------------------------

class SearchMovement(object):
    """
    this class asks user search parameters.
    if there are movements with these attributes, they're shown in
    a DataForm object, else a tkMessageBox warns that there aren't movements.
    """
    def __init__(self, master, controller):
        self.controller=controller

        #self.toplevel=tk.Toplevel(master)
        self.toplevel=master
        self.valueLabel=tk.Label(self.toplevel, text="inserisci valore del movimento")
        self.valueLabel.pack()
        self.valueEntry=tk.Entry(self.toplevel)
        self.valueEntry.pack()

        self.dateLabel=tk.Label(self.toplevel, text="inserisci la data del movimento\nformato: dd/mm/yyyy")
        self.dateLabel.pack()
        self.dateEntry=tk.Entry(self.toplevel)
        self.dateEntry.pack()

        self.notesLabel=tk.Label(self.toplevel, text="inserisci la nota che cerchi")
        self.notesLabel.pack()
        self.notesEntry=tk.Entry(self.toplevel)
        self.notesEntry.pack()

        self.searchButton=tk.Button(self.toplevel, text="cerca movimento", command=self.searchMovement)
        self.searchButton.pack()
        #questa variabile serve per mantenere l'indice della listbox causalsListbox in cui � indicato la direzione del movimento
        self.causalIndex='*'


    def searchMovement(self):
        import re
        value=unicode(self.valueEntry.get())
        if value=="": value="*"

        pattern=re.compile("\d{1,2}/\d{1,2}/\d{4}")
        date=str(self.dateEntry.get())
        if date=="": day, month, year='*'*3
        else:
            if re.match(pattern, date):
                day, month, year=(int(item) for item in date.split('/'))
            else:
                print "no matches"
                msgBox=tkMessageBox.showwarning(title="errore nella data", message="errore nella data.\n\
                il formato e\' dd/mm/yyyy")


        notes=unicode(self.notesEntry.get())
        if notes=="": notes="*"

        movements=self.controller.get_movement(value=value, notes=notes, \
        year=year,month=month, day=day)

        if not movements: tkMessageBox.showwarning(title='search movement', message='non ci sono movimenti corrispondenti ai parametri dati')
        else:
            dataform=DataForm(self.toplevel, 8)
            dataform.addRow(['ID', 'valore', 'causale', 'notes', 'direzione', 'anno', 'mese', 'giorno'])
            for movement in movements:
                dataform.addRow(movement)


#------------- LastMovementForm class ----------------------------------

class LastMovementForm(object):
    """
    this form is only a tkMessageBox that
    show the date of the last inserted movement.
    """
    def __init__(self, controller):
        lastMov=self.get_last_movement(controller)
        try:
            t="L'ultimo movimento � stato fatto il %i/%i/%i"%(
            lastMov[0], lastMov[1], lastMov[2])
            tkMessageBox.showinfo(title="ultimo movimento", message=t)
        except TypeError:
            #trying to visualize a None instead of an integer
            tkMessageBox.showerror(title="errore: ultimo movimento",
                message="non sono stati trovati movimenti.")

    def get_last_movement(self, controller):
        """
        Questa funzione permette di ottenere
         l'ultimo movimento in ordine cronologico.
        """
        lastMovement=controller.get_last_movement()
        try:
            day, month, year=lastMovement
        except TypeError:
            day, month, year=None, None, None
        return [day, month, year]


#------------- SearchUpdateForm class ----------------------------------

class SearchUpdateForm(SearchMovement):
    """
    this class asks user parameter for search a movement.
    this movement will be modified
    """
    def __init__(self,  master, model):
        self.model=model

        self.toplevel=tk.Toplevel(master)

        SearchMovement.__init__(self, self.toplevel, self.model)

        self.idEntry=tk.Entry(self.toplevel)
        self.idEntry.pack()
        self.openUpdateForm=tk.Button(self.toplevel,
            text="modifica il movimento",
            command=self.openUpdateFormFunction
            )
        self.openUpdateForm.pack()

    def openUpdateFormFunction(self):
        if self.model.get_movement_by_id(int(self.idEntry.get())):
            UpdateForm(self.toplevel, self.model, int(self.idEntry.get()))
        else:
            tkMessageBox.showerror(message="errore: l'ID inserito non � valido. Reinserire.",
                title="errore ID")

#------------- UpdateForm class ----------------------------------

class UpdateForm(object):
    """
    this class is the form trought the user
    can modify a movement.
    """
    def __init__(self, toplevel, model, movement_id):
        self.toplevel=tk.Toplevel(toplevel)
        self.mov_id=movement_id
        self.model=model
        #type of movement is a tuple with all attributes.
        self.movement=self.model.get_movement_by_id(self.mov_id)[0]

        #position 1 is value (2nd position of the list)
        self.valueLabel=tk.Label(self.toplevel,
            text="valore precedente:\n"+unicode(self.movement[self.model.VALUE])
            )
        self.valueLabel.pack()
        self.valueEntry=tk.Entry(self.toplevel)
        self.valueEntry.pack()

        #position 2 is causal
        self.causalLabel=tk.Label(self.toplevel,
            text="causale precedente:\n"+unicode(self.movement[self.model.CAUSAL])
            )
        self.causalLabel.pack()
        #insert a listbox with all causals
        #self.causalEntry=tk.Entry(self.toplevel)
        #self.causalEntry.pack()

        #position 3 is notes
        self.notesLabel=tk.Label(self.toplevel,
            text="nota precedente:\n"+unicode(self.movement[self.model.NOTES])
            )
        self.notesLabel.pack()
        self.notesEntry=tk.Entry(self.toplevel)
        self.notesEntry.pack()

        #position 4 is direction and we can't modify it.
        #so, position 5 is year
        self.yearLabel=tk.Label(self.toplevel,
            text="anno precedente:\n"+unicode(self.movement[self.model.YEAR])
            )
        self.yearLabel.pack()
        self.yearEntry=tk.Entry(self.toplevel)
        self.yearEntry.pack()

        #position 6 is month
        self.monthLabel=tk.Label(self.toplevel,
            text="mese precedente:\n"+unicode(self.movement[self.model.MONTH])
            )
        self.monthLabel.pack()
        self.monthEntry=tk.Entry(self.toplevel)
        self.monthEntry.pack()

        #position 7 is day
        self.dayLabel=tk.Label(self.toplevel,
            text="giorno precedente:\n"+unicode(self.movement[self.model.DAY])
            )
        self.dayLabel.pack()
        self.dayEntry=tk.Entry(self.toplevel)
        self.dayEntry.pack()

        #and now... a button to modify all!
        self.updateButton=tk.Button(self.toplevel,
            text="aggiorna",
            command=self.update
            )
        self.updateButton.pack()

    def update(self):
        """
        this method takes all values put in entry widgets.
        if the entry is void, default value is actual value.
        """
        valuesDict={}
        valuesDict['value']=(self.valueEntry.get()!="" and self.valueEntry.get()) or \
            self.movement[self.model.VALUE]
        valuesDict['notes']=(self.notesEntry.get()!="" and self.notesEntry.get()) or \
            self.movement[self.model.NOTES]
        valuesDict['year']=(self.yearEntry.get()!="" and self.yearEntry.get()) or \
            self.movement[self.model.YEAR]
        valuesDict['month']=(self.monthEntry.get()!="" and self.monthEntry.get()) or \
            self.movement[self.model.MONTH]
        valuesDict['day']=(self.dayEntry.get()!="" and self.dayEntry.get()) or \
            self.movement[self.model.DAY]
        self.model.update_movement(self.mov_id, valuesDict)
        tkMessageBox.showinfo(title="dati aggiornati", message="ok, dati aggiornati correttamente")

#------------- main! ----------------------------------

if __name__=="__main__":
    root=tk.Tk()
    v=View(root, Controller(), Model())
    root.mainloop()
